@@ -1,3 +1,4 @@
+import typing
 import shioaji as sj
 from touchprice.core import Base
 from touchprice.constant import Trend, PriceType
@@ -44,6 +45,8 @@ class TouchCmd(Base):
     low: Price = None
     volume: Qty = None
     total_volume: Qty = None
+    ask_volume: Qty = None
+    bid_volume: Qty = None
 
     def __init__(
         self,
@@ -55,6 +58,8 @@ class TouchCmd(Base):
         low: Price = None,
         volume: Qty = None,
         total_volume: Qty = None,
+        ask_volume: Qty = None,
+        bid_volume: Qty = None,
     ):
         super().__init__(
             **dict(
@@ -66,6 +71,8 @@ class TouchCmd(Base):
                 low=low,
                 volume=volume,
                 total_volume=total_volume,
+                ask_volume=ask_volume,
+                bid_volume=bid_volume,
             )
         )
 
@@ -78,12 +85,40 @@ class OrderCmd(Base):
         super().__init__(**dict(code=code, order=order))
 
 
+class LossProfitCmd(Base):
+    loss_pricegap: PriceGap = None
+    loss_order: sj.Order = None
+    profit_pricegap: PriceGap = None
+    profit_order: sj.Order = None
+
+
+class StoreLossProfit(Base):
+    loss_close: PriceGap = None
+    profit_close: PriceGap = None
+    order_contract: sj.contracts.Contract
+    loss_order: sj.Order = None
+    profit_order: sj.Order = None
+    result: sj.order.Trade = None
+    excuted_cb: Callable[[sj.order.Trade], sj.order.Trade] = print
+    excuted: bool = False
+
+
 class TouchOrderCond(Base):
     touch_cmd: TouchCmd
     order_cmd: OrderCmd
+    lossprofit_cmd: LossProfitCmd = None
 
-    def __init__(self, touch_cmd: TouchCmd, order_cmd: OrderCmd):
-        super().__init__(**dict(touch_cmd=touch_cmd, order_cmd=order_cmd))
+    def __init__(
+        self,
+        touch_cmd: TouchCmd,
+        order_cmd: OrderCmd,
+        lossprofit_cmd: typing.List[LossProfitCmd] = None,
+    ):
+        super().__init__(
+            **dict(
+                touch_cmd=touch_cmd, order_cmd=order_cmd, lossprofit_cmd=lossprofit_cmd,
+            )
+        )
 
 
 class StoreCond(Base):
@@ -94,6 +129,8 @@ class StoreCond(Base):
     low: PriceGap = None
     volume: QtyGap = None
     total_volume: QtyGap = None
+    ask_volume: QtyGap = None
+    bid_volume: QtyGap = None
     order_contract: sj.contracts.Contract
     order: sj.Order
     result: sj.order.Trade = None
@@ -111,3 +148,6 @@ class StatusInfo(Base):
     change_rate: float  # 幅度
     volume: int
     total_volume: int
+    ask_volume: int = 0
+    bid_volume: int = 0
+    add_ts: int = 0
\ No newline at end of file
