@@ -1,5 +1,6 @@
 import shioaji as sj
 import typing
+import datetime
 from pydantic import StrictInt
 from functools import partial
 from touchprice.constant import Trend, PriceType
@@ -13,6 +14,8 @@ from touchprice.condition import (
     StatusInfo,
     Qty,
     QtyGap,
+    LossProfitCmd,
+    StoreLossProfit,
 )
 
 
@@ -28,15 +31,23 @@ def get_contracts(api: sj.Shioaji):
 class TouchOrderExecutor:
     def __init__(self, api: sj.Shioaji):
         self.api: sj.Shioaji = api
-        self.conditions: typing.Dict[str, typing.List[StoreCond]] = {}
+        self.conditions: typing.Dict[
+            str, typing.List[typing.Union[StoreLossProfit, StoreCond]]
+        ] = {}
         self.infos: typing.Dict[str, StatusInfo] = {}
         self.contracts: dict = get_contracts(self.api)
         self.api.quote.set_quote_callback(self.integration)
+        self.orders: typing.Dict[str, typing.Dict[str, StoreLossProfit]] = {}
 
     def update_snapshot(self, contract: sj.contracts.Contract):
         code = contract.code
         if code not in self.infos.keys():
-            self.infos[code] = StatusInfo(**self.api.snapshots([contract])[0])
+            snapshot = self.api.snapshots([contract])[0]
+            self.infos[code] = StatusInfo(**snapshot)
+            now = datetime.datetime.now(datetime.timezone.utc)
+            add_ts = now.timestamp()
+            self.infos[code].add_ts = add_ts
+            volume = self.infos[code].volume
 
     @staticmethod
     def set_price(price_info: Price, contract: sj.contracts.Contract):
@@ -55,12 +66,14 @@ class TouchOrderExecutor:
         tconds_dict.pop("code")
         if tconds_dict:
             for key, value in tconds_dict.items():
-                if key not in ["volume", "total_volume"]:
+                if key not in ["volume", "total_volume", "ask_volume", "bid_volume"]:
                     tconds_dict[key] = TouchOrderExecutor.set_price(
                         Price(**value), contract
                     )
             tconds_dict["order_contract"] = self.contracts[condition.order_cmd.code]
             tconds_dict["order"] = condition.order_cmd.order
+            if condition.lossprofit_cmd:
+                tconds_dict["excuted_cb"] = place_order_cb
             return StoreCond(**tconds_dict)
 
     def add_condition(self, condition: TouchOrderCond):
@@ -105,20 +118,49 @@ class TouchOrderExecutor:
             info = self.infos[code].dict()
             for num, conds in enumerate(conditions):
                 if not conds.excuted:
-                    order = conds.order
                     order_contract = conds.order_contract
-                    cond = conds.dict()
-                    cond.pop("order")
-                    cond.pop("order_contract")
-                    cond.pop("excuted")
-                    cond.pop("excuted_cb")
-                    if all(
-                        self.touch_cond(value, info[key]) for key, value in cond.items()
-                    ):
-                        self.conditions[code][num].excuted = True
-                        self.conditions[code][num].result = self.api.place_order(
-                            order_contract, order, cb=self.conditions[code][num].excuted_cb
-                        )
+                    if isinstance(conds, StoreCond):
+                        order = conds.order
+                        cond = conds.dict()
+                        cond.pop("order")
+                        cond.pop("order_contract")
+                        cond.pop("excuted")
+                        cond.pop("excuted_cb")
+                        if all(
+                            self.touch_cond(value, info[key])
+                            for key, value in cond.items()
+                        ):
+                            self.conditions[code][num].excuted = True
+                            self.conditions[code][num].result = self.api.place_order(
+                                order_contract,
+                                order,
+                                cb=self.conditions[code][num].excuted_cb,
+                            )
+                    elif isinstance(conds, StoreLossProfit):
+                        loss_order = conds.loss_order
+                        profit_order = conds.profit_order
+                        cond = conds.dict()
+                        cond.pop("loss_order")
+                        cond.pop("profit_order")
+                        cond.pop("order_contract")
+                        cond.pop("excuted")
+                        cond.pop("excuted_cb")
+                        order = None
+                        if "loss_close" in cond and self.touch_cond(
+                            cond["loss_close"], info["close"]
+                        ):
+                            order = loss_order
+                        elif "profit_close" in cond and self.touch_cond(
+                            cond["profit_close"], info["close"]
+                        ):
+                            order = profit_order
+                        if order:
+                            self.conditions[code][num].excuted = True
+                            self.conditions[code][num].result = self.api.place_order(
+                                order_contract,
+                                order,
+                                cb=self.conditions[code][num].excuted_cb,
+                            )
 
     def integration(self, topic: str, quote: dict):
         if "Simtrade" in quote.keys():
@@ -132,6 +174,20 @@ class TouchOrderExecutor:
                 info.low = info.close if info.low > info.close else info.low
                 info.total_volume = quote["VolSum"][0]
                 info.volume = quote["Volume"][0]
+                if quote["TickType"][0] == 1:
+                    info.ask_volume = (
+                        info.ask_volume + info.volume
+                        if info.ask_volume
+                        else info.volume
+                    )
+                    info.bid_volume = 0
+                elif quote["TickType"][0] == 2:
+                    info.bid_volume = (
+                        info.bid_volume + info.volume
+                        if info.bid_volume
+                        else info.volume
+                    )
+                    info.ask_volume = 0
                 self.touch(code)
         elif topic.startswith("QUT/") or topic.startswith("Q/"):
             code = topic.split("/")[-1]
@@ -147,3 +203,29 @@ class TouchOrderExecutor:
             return self.conditions
         else:
             return self.conditions[code]
+
+    def place_order_cb(self, state, msg):
+        if "Order" in state:
+            code = msg["contract"]["code"]
+            conds = self.conditions[code] if code in self.conditions else []
+            for cond in conds:
+                if cond.result == msg:
+                    seqno = msg["order"]["seqno"]
+                    price = float(msg["order"]["price"])
+                    storecond = StoreLossProfit(
+                        loss_close=cond.lossprofit_cmd.loss_pricegap,
+                        profit_close=cond.lossprofit_cmd.profit_pricegap,
+                        order_contract=msg["contract"],
+                        loss_order=cond.lossprofit_cmd.lossorder_cmd,
+                        profit_order=cond.lossprofit_cmd.profitorder_cmd,
+                    )
+                    self.orders[seqno][code] = storecond
+        elif "Deal" in state:
+            seqno = msg["seqno"]
+            if self.orders[seqno]:
+                code = msg["code"]
+                store_cond = self.orders[seqno][code]
+                if code in self.conditions.keys():
+                    self.conditions[code].append(store_cond)
+                else:
+                    self.conditions[code] = [store_cond]
